<!DOCTYPE html>
<html>
<head>
    <title>fossil</title>

    <meta name="viewport" content="width=device-width, initial-scale=1.0">
    <link rel="stylesheet" href="/static/style.css">
    <script src="https://unpkg.com/htmx.org@1.9.10" integrity="sha384-D1Kt99CQMDuVetoL1lrYwg5t+9QdHe7NLX/SoJYkXDFfX37iInKRy5xLSi8nO7UC" crossorigin="anonymous"></script>
    <script src="https://unpkg.com/htmx.org/dist/ext/json-enc.js"></script>
    <script src="/static/page.js"></script>
    {% for item in head_html() %}
        {% autoescape false %}
        {{ item }}
        {% endautoescape %}
    {% endfor %}
</head>
<body>
    <h1 class="nav">
        <div>
            <img src="/static/logo-light.svg" class="logo" />
        </div>
        <div class="extra-nav">
            {% for item in extra_nav() %}
                {% autoescape false %}
                {{ item.html }}
                {% endautoescape %}
            {% endfor %}
            <button onclick="htmx.toggleClass(htmx.find('#hamburger'), 'open')" class="hamburger-launch">
                <svg height="32px" id="Layer_1" style="enable-background:new 0 0 32 32;" version="1.1" viewBox="0 0 32 32" width="32px" xml:space="preserve" xmlns="http://www.w3.org/2000/svg" xmlns:xlink="http://www.w3.org/1999/xlink"><path d="M4,10h24c1.104,0,2-0.896,2-2s-0.896-2-2-2H4C2.896,6,2,6.896,2,8S2.896,10,4,10z M28,14H4c-1.104,0-2,0.896-2,2  s0.896,2,2,2h24c1.104,0,2-0.896,2-2S29.104,14,28,14z M28,22H4c-1.104,0-2,0.896-2,2s0.896,2,2,2h24c1.104,0,2-0.896,2-2  S29.104,22,28,22z"/></svg>
            </button> 
        </div>
    </h1>
    <div id="hamburger" class="hamburger">
        <a href="/">Home</a>
        <a href="/settings">Settings</a>
        {% for menu_item in extra_menu_items() %}
            {% autoescape false %}
            {{ menu_item.html }}
            {% endautoescape %}
        {% endfor %}
    </div>
    <div>
        {% block content %}
        <!-- Specialized content goes here -->
        {% endblock %}
    </div>
    
    <footer>
        <!-- <p>© 2023 Tim Kellogg. All rights reserved.</p> -->
    </footer>
</body>
</html>
